/**
 * Bundle of @devexpress/dx-react-grid-bootstrap3
 * Generated: 2017-06-09
 * Version: 1.0.0-alpha.2
 * License: https://js.devexpress.com/Licensing
 */

import React from 'react';
import PropTypes from 'prop-types';
import { DragDropContext, Grid, GroupingPanel, PagingPanel, TableEditColumn, TableEditRow, TableFilterRow, TableGroupRow, TableHeaderRow, TableLayout, TableRowDetail, TableSelection, TableView } from '@devexpress/dx-react-grid';
import { Pager, Pagination } from 'react-bootstrap';
import { firstRowOnPage, lastRowOnPage } from '@devexpress/dx-grid-core';
import { DragSource, combineTemplates } from '@devexpress/dx-react-core';

var Root = function Root(_ref) {
  var headerTemplate = _ref.headerTemplate,
      bodyTemplate = _ref.bodyTemplate,
      footerTemplate = _ref.footerTemplate;
  return React.createElement(
    'div',
    { className: 'panel panel-default' },
    headerTemplate(),
    bodyTemplate(),
    footerTemplate()
  );
};

Root.propTypes = {
  headerTemplate: PropTypes.func.isRequired,
  bodyTemplate: PropTypes.func.isRequired,
  footerTemplate: PropTypes.func.isRequired
};

var Header = function Header(_ref2) {
  var children = _ref2.children;
  return children && React.createElement(
    'div',
    { className: 'panel-heading' },
    children
  );
};

Header.propTypes = {
  children: PropTypes.node
};

Header.defaultProps = {
  children: null
};

var Footer = function Footer(_ref3) {
  var children = _ref3.children;
  return children && React.createElement(
    'div',
    { className: 'panel-footer' },
    children
  );
};

Footer.propTypes = {
  children: PropTypes.node
};

Footer.defaultProps = {
  children: null
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();





var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};



var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};









var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var Grid$1 = function Grid$$1(_ref) {
  var children = _ref.children,
      props = objectWithoutProperties(_ref, ['children']);
  return React.createElement(
    Grid,
    _extends({
      rootTemplate: Root,
      headerPlaceholderTemplate: Header,
      footerPlaceholderTemplate: Footer
    }, props),
    children
  );
};

Grid$1.propTypes = {
  children: PropTypes.oneOfType([PropTypes.arrayOf(PropTypes.node), PropTypes.node]).isRequired
};

var Container = function Container(_ref) {
  var clientOffset = _ref.clientOffset,
      columns = _ref.columns,
      columnTemplate = _ref.columnTemplate;
  return React.createElement(
    'ul',
    {
      className: 'list-group',
      style: {
        position: 'fixed',
        zIndex: 1000,
        left: 0,
        top: 0,
        display: 'inline-block',
        transform: 'translate(calc(' + clientOffset.x + 'px - 50%), calc(' + clientOffset.y + 'px - 50%))'
      }
    },
    columns.map(function (column) {
      return React.cloneElement(columnTemplate({ column: column }), { key: column.name });
    })
  );
};

Container.propTypes = {
  clientOffset: PropTypes.shape({
    x: PropTypes.number.isRequired,
    y: PropTypes.number.isRequired
  }).isRequired,
  columns: PropTypes.array.isRequired,
  columnTemplate: PropTypes.func.isRequired
};

var Column = function Column(_ref2) {
  var column = _ref2.column;
  return React.createElement(
    'li',
    { className: 'list-group-item' },
    column.title
  );
};

Column.propTypes = {
  column: PropTypes.shape().isRequired
};

var DragDropContext$1 = function DragDropContext$$1(props) {
  return React.createElement(DragDropContext, _extends({
    containerTemplate: Container,
    columnTemplate: Column
  }, props));
};

var PageSizeSelector = function PageSizeSelector(_ref) {
  var pageSize = _ref.pageSize,
      onPageSizeChange = _ref.onPageSizeChange,
      allowedPageSizes = _ref.allowedPageSizes;
  return React.createElement(
    'div',
    { style: { display: 'inline-block' } },
    React.createElement(
      'select',
      {
        className: 'form-control visible-xs-inline-block',
        style: { width: 'auto' },
        value: pageSize,
        onChange: function onChange(e) {
          return onPageSizeChange(parseInt(e.target.value, 10));
        }
      },
      allowedPageSizes.map(function (val) {
        return React.createElement(
          'option',
          { key: val, value: val },
          val
        );
      })
    ),
    React.createElement(
      'ul',
      {
        className: 'pagination hidden-xs',
        style: {
          margin: 0,
          verticalAlign: 'bottom'
        }
      },
      allowedPageSizes.map(function (item) {
        return React.createElement(
          'li',
          { key: item, className: item === pageSize ? 'active' : '' },
          React.createElement(
            'a',
            {
              href: '#',
              onClick: function onClick(e) {
                e.preventDefault();
                onPageSizeChange(item);
              }
            },
            item
          )
        );
      })
    )
  );
};

PageSizeSelector.propTypes = {
  pageSize: PropTypes.number.isRequired,
  onPageSizeChange: PropTypes.func.isRequired,
  allowedPageSizes: PropTypes.arrayOf(PropTypes.number).isRequired
};

var Pager$1 = function Pager$$1(_ref) {
  var currentPage = _ref.currentPage,
      onCurrentPageChange = _ref.onCurrentPageChange,
      totalPages = _ref.totalPages,
      pageSize = _ref.pageSize,
      onPageSizeChange = _ref.onPageSizeChange,
      allowedPageSizes = _ref.allowedPageSizes,
      totalCount = _ref.totalCount;
  return React.createElement(
    'div',
    { className: 'clearfix' },
    !!allowedPageSizes.length && React.createElement(PageSizeSelector, {
      pageSize: pageSize,
      onPageSizeChange: onPageSizeChange,
      allowedPageSizes: allowedPageSizes
    }),
    React.createElement(Pagination, {
      style: {
        margin: 0,
        verticalAlign: 'bottom'
      },
      className: 'pull-right hidden-xs',
      items: totalPages,
      activePage: currentPage + 1,
      boundaryLinks: true,
      maxButtons: 3,
      onSelect: function onSelect(page) {
        return onCurrentPageChange(page - 1);
      }
    }),
    React.createElement(
      Pager,
      {
        className: 'pull-right visible-xs',
        style: { margin: 0 }
      },
      React.createElement(
        Pager.Item,
        {
          disabled: currentPage === 0,
          onClick: function onClick() {
            return onCurrentPageChange(currentPage - 1);
          }
        },
        '\xAB'
      ),
      ' ',
      React.createElement(
        Pager.Item,
        {
          disabled: currentPage === totalPages - 1,
          onClick: function onClick() {
            return onCurrentPageChange(currentPage + 1);
          }
        },
        '\xBB'
      )
    ),
    React.createElement(
      'span',
      { className: 'pull-right visible-xs', style: { marginRight: '20px' } },
      React.createElement(
        'span',
        { style: { display: 'inline-block', verticalAlign: 'middle', lineHeight: '32px' } },
        String(firstRowOnPage(currentPage, pageSize)),
        '-',
        String(lastRowOnPage(currentPage, pageSize, totalCount)),
        ' of ',
        String(totalCount)
      )
    )
  );
};

Pager$1.propTypes = {
  currentPage: PropTypes.number.isRequired,
  onCurrentPageChange: PropTypes.func.isRequired,
  totalPages: PropTypes.number.isRequired,
  pageSize: PropTypes.number.isRequired,
  onPageSizeChange: PropTypes.func.isRequired,
  allowedPageSizes: PropTypes.arrayOf(PropTypes.number).isRequired,
  totalCount: PropTypes.number.isRequired
};

var pagerTemplate = function pagerTemplate(props) {
  return React.createElement(Pager$1, props);
};

var PagingPanel$1 = function PagingPanel$$1(props) {
  return React.createElement(PagingPanel, _extends({
    pagerTemplate: pagerTemplate
  }, props));
};

var GroupPanel = function GroupPanel(_ref) {
  var groupedColumns = _ref.groupedColumns,
      groupByColumnText = _ref.groupByColumnText,
      cellTemplate = _ref.cellTemplate;

  var text = function text() {
    return groupByColumnText || React.createElement(
      'span',
      null,
      'Click \xA0',
      React.createElement('i', {
        className: 'glyphicon glyphicon-th-list',
        style: {
          top: '0',
          fontSize: '9px'
        }
      }),
      '\xA0 icon in the column header to group by that column'
    );
  };

  var GroupPanelCell = cellTemplate;

  return groupedColumns.length ? React.createElement(
    'div',
    null,
    groupedColumns.map(function (column) {
      return React.createElement(GroupPanelCell, {
        key: column.name,
        column: column
      });
    })
  ) : React.createElement(
    'span',
    null,
    text()
  );
};

GroupPanel.propTypes = {
  groupedColumns: PropTypes.array.isRequired,
  groupByColumnText: PropTypes.string,
  cellTemplate: PropTypes.func.isRequired
};

GroupPanel.defaultProps = {
  groupByColumnText: undefined
};

var SortingIndicator = function SortingIndicator(_ref) {
  var direction = _ref.direction,
      style = _ref.style;
  return React.createElement('i', {
    className: 'glyphicon glyphicon-arrow-' + (direction === 'desc' ? 'down' : 'up'),
    style: _extends({
      top: '0',
      fontSize: '9px'
    }, style)
  });
};

SortingIndicator.propTypes = {
  direction: PropTypes.oneOf(['asc', 'desc']),
  style: PropTypes.shape()
};

SortingIndicator.defaultProps = {
  direction: null,
  style: null
};

var GroupPanelCell = function GroupPanelCell(_ref) {
  var column = _ref.column,
      groupByColumn = _ref.groupByColumn,
      allowSorting = _ref.allowSorting,
      sortingDirection = _ref.sortingDirection,
      changeSortingDirection = _ref.changeSortingDirection;
  return React.createElement(
    'div',
    {
      className: 'btn btn-default',
      style: {
        marginRight: '5px'
      }
    },
    React.createElement(
      'span',
      {
        onClick: function onClick(e) {
          if (!allowSorting) return;
          var cancelSortingRelatedKey = e.metaKey || e.ctrlKey;
          changeSortingDirection({
            keepOther: e.shiftKey || cancelSortingRelatedKey,
            cancel: cancelSortingRelatedKey
          });
        }
      },
      column.title || column.name,
      allowSorting && sortingDirection && React.createElement(
        'span',
        null,
        '\xA0',
        React.createElement(SortingIndicator, {
          direction: sortingDirection
        })
      )
    ),
    '\xA0',
    React.createElement('i', {
      className: 'glyphicon glyphicon-remove',
      style: {
        top: '0',
        fontSize: '9px',
        margin: '-5px',
        padding: '5px'
      },
      onClick: function onClick() {
        return groupByColumn({ columnName: column.name });
      }
    })
  );
};

GroupPanelCell.defaultProps = {
  allowSorting: false,
  sortingDirection: undefined,
  changeSortingDirection: undefined,
  allowGrouping: false,
  groupByColumn: undefined
};

GroupPanelCell.propTypes = {
  column: PropTypes.shape({
    title: PropTypes.string
  }).isRequired,
  allowSorting: PropTypes.bool,
  sortingDirection: PropTypes.oneOf(['asc', 'desc', null]),
  changeSortingDirection: PropTypes.func,
  groupByColumn: PropTypes.func
};

var groupPanelCellTemplate = function groupPanelCellTemplate(props) {
  return React.createElement(GroupPanelCell, props);
};

var GroupingPanel$1 = function GroupingPanel$$1(_ref) {
  var groupByColumnText = _ref.groupByColumnText,
      restProps = objectWithoutProperties(_ref, ['groupByColumnText']);
  return React.createElement(GroupingPanel, _extends({
    groupPanelTemplate: function groupPanelTemplate(props) {
      return React.createElement(GroupPanel, _extends({
        groupByColumnText: groupByColumnText
      }, props));
    },
    groupPanelCellTemplate: groupPanelCellTemplate
  }, restProps));
};

GroupingPanel$1.propTypes = {
  allowSorting: PropTypes.bool,
  groupByColumnText: PropTypes.string
};
GroupingPanel$1.defaultProps = {
  allowSorting: false,
  groupByColumnText: undefined
};

var TableDetailToggleCell = function TableDetailToggleCell(_ref) {
  var style = _ref.style,
      expanded = _ref.expanded,
      toggleExpanded = _ref.toggleExpanded;
  return React.createElement(
    'td',
    {
      style: _extends({
        cursor: 'pointer'
      }, style),
      onClick: function onClick(e) {
        e.stopPropagation();
        toggleExpanded();
      }
    },
    React.createElement('i', {
      className: 'glyphicon glyphicon-triangle-' + (expanded ? 'bottom' : 'right'),
      style: {
        fontSize: '9px',
        top: '0'
      }
    })
  );
};

TableDetailToggleCell.propTypes = {
  style: PropTypes.shape(),
  expanded: PropTypes.bool,
  toggleExpanded: PropTypes.func
};

TableDetailToggleCell.defaultProps = {
  style: null,
  expanded: false,
  toggleExpanded: function toggleExpanded() {}
};

var TableDetailCell = function TableDetailCell(_ref) {
  var colspan = _ref.colspan,
      style = _ref.style,
      template = _ref.template;
  return React.createElement(
    'td',
    { style: style, colSpan: colspan, className: 'active' },
    template()
  );
};

TableDetailCell.propTypes = {
  style: PropTypes.shape(),
  colspan: PropTypes.number,
  template: PropTypes.func.isRequired
};

TableDetailCell.defaultProps = {
  style: null,
  colspan: 1
};

var detailToggleTemplate = function detailToggleTemplate(props) {
  return React.createElement(TableDetailToggleCell, props);
};
var detailCellTemplate = function detailCellTemplate(props) {
  return React.createElement(TableDetailCell, props);
};

var TableRowDetail$1 = function TableRowDetail$$1(props) {
  return React.createElement(TableRowDetail, _extends({
    detailToggleTemplate: detailToggleTemplate,
    detailCellTemplate: detailCellTemplate,
    detailToggleCellWidth: 25
  }, props));
};

var TableGroupRowCell = function TableGroupRowCell(_ref) {
  var style = _ref.style,
      colspan = _ref.colspan,
      row = _ref.row,
      isExpanded = _ref.isExpanded,
      toggleGroupExpanded = _ref.toggleGroupExpanded;
  return React.createElement(
    'td',
    {
      colSpan: colspan,
      style: _extends({
        cursor: 'pointer'
      }, style),
      onClick: toggleGroupExpanded
    },
    React.createElement('i', {
      className: 'glyphicon glyphicon-triangle-' + (isExpanded ? 'bottom' : 'right'),
      style: {
        fontSize: '9px',
        top: 0,
        marginRight: '10px'
      }
    }),
    React.createElement(
      'strong',
      null,
      row.column.title || row.column.name,
      ': ',
      row.value
    )
  );
};

TableGroupRowCell.propTypes = {
  style: PropTypes.shape(),
  colspan: PropTypes.number,
  row: PropTypes.shape(),
  isExpanded: PropTypes.bool,
  toggleGroupExpanded: PropTypes.func
};

TableGroupRowCell.defaultProps = {
  style: null,
  colspan: 1,
  row: {},
  isExpanded: false,
  toggleGroupExpanded: function toggleGroupExpanded() {}
};

var TableGroupIndentCell = function TableGroupIndentCell(_ref2) {
  var column = _ref2.column,
      style = _ref2.style;
  return React.createElement(
    'td',
    {
      style: _extends({}, style, {
        width: column.width
      })
    },
    '\xA0'
  );
};

TableGroupIndentCell.propTypes = {
  style: PropTypes.object,
  column: PropTypes.shape({
    width: PropTypes.number.isRequired
  }).isRequired
};

TableGroupIndentCell.defaultProps = {
  style: {}
};

var groupRowCellTemplate = function groupRowCellTemplate(props) {
  return React.createElement(TableGroupRowCell, props);
};
var groupIndentCellTemplate = function groupIndentCellTemplate(props) {
  return React.createElement(TableGroupIndentCell, props);
};

var TableGroupRow$1 = function TableGroupRow$$1(props) {
  return React.createElement(TableGroupRow, _extends({
    groupRowCellTemplate: groupRowCellTemplate,
    groupIndentCellTemplate: groupIndentCellTemplate,
    groupColumnWidth: 20
  }, props));
};

var TableSelectAllCell = function TableSelectAllCell(_ref) {
  var style = _ref.style,
      allSelected = _ref.allSelected,
      someSelected = _ref.someSelected,
      selectionAvailable = _ref.selectionAvailable,
      toggleAll = _ref.toggleAll;
  return React.createElement(
    'th',
    {
      style: _extends({
        cursor: selectionAvailable && 'pointer'
      }, style),
      onClick: function onClick(e) {
        if (!selectionAvailable) return;

        e.stopPropagation();
        toggleAll();
      }
    },
    React.createElement('input', {
      style: {
        cursor: selectionAvailable && 'pointer',
        margin: 0
      },
      type: 'checkbox',
      disabled: !selectionAvailable,
      checked: allSelected,
      ref: function ref(_ref2) {
        if (_ref2) {
          var checkbox = _ref2;
          checkbox.indeterminate = someSelected;
        }
      },
      onChange: toggleAll,
      onClick: function onClick(e) {
        return e.stopPropagation();
      }
    })
  );
};
TableSelectAllCell.defaultProps = {
  style: null,
  allSelected: false,
  someSelected: false,
  selectionAvailable: false,
  toggleAll: function toggleAll() {}
};
TableSelectAllCell.propTypes = {
  style: PropTypes.shape(),
  allSelected: PropTypes.bool,
  someSelected: PropTypes.bool,
  selectionAvailable: PropTypes.bool,
  toggleAll: PropTypes.func
};

var TableSelectCell = function TableSelectCell(_ref) {
  var style = _ref.style,
      selected = _ref.selected,
      changeSelected = _ref.changeSelected;
  return React.createElement(
    'td',
    {
      style: _extends({
        cursor: 'pointer'
      }, style),
      onClick: function onClick(e) {
        e.stopPropagation();
        changeSelected();
      }
    },
    React.createElement('input', {
      style: {
        cursor: 'pointer',
        margin: 0
      },
      type: 'checkbox',
      checked: selected,
      onChange: changeSelected,
      onClick: function onClick(e) {
        return e.stopPropagation();
      }
    })
  );
};
TableSelectCell.defaultProps = {
  style: null,
  selected: false,
  changeSelected: function changeSelected() {}
};
TableSelectCell.propTypes = {
  style: PropTypes.shape(),
  selected: PropTypes.bool,
  changeSelected: PropTypes.func
};

var selectCellTemplate = function selectCellTemplate(props) {
  return React.createElement(TableSelectCell, props);
};
var selectAllCellTemplate = function selectAllCellTemplate(props) {
  return React.createElement(TableSelectAllCell, props);
};

var TableSelection$1 = function TableSelection$$1(props) {
  return React.createElement(TableSelection, _extends({
    selectCellTemplate: selectCellTemplate,
    selectAllCellTemplate: selectAllCellTemplate,
    selectionColumnWidth: 30
  }, props));
};

var MINIMAL_COLUMN_WIDTH = 120;

/* eslint-disable react/prop-types */
var tableTemplate$1 = function tableTemplate(_ref) {
  var children = _ref.children,
      restProps = objectWithoutProperties(_ref, ['children']);
  return React.createElement(
    'table',
    _extends({
      className: 'table'
    }, restProps),
    children
  );
};
var headTemplate = function headTemplate(_ref2) {
  var children = _ref2.children,
      restProps = objectWithoutProperties(_ref2, ['children']);
  return React.createElement(
    'thead',
    restProps,
    children
  );
};
var bodyTemplate = function bodyTemplate(_ref3) {
  var children = _ref3.children,
      restProps = objectWithoutProperties(_ref3, ['children']);
  return React.createElement(
    'tbody',
    restProps,
    children
  );
};
var rowTemplate = function rowTemplate(_ref4) {
  var children = _ref4.children,
      row = _ref4.row,
      restProps = objectWithoutProperties(_ref4, ['children', 'row']);
  return React.createElement(
    'tr',
    _extends({
      className: row.selected ? 'active' : ''
    }, restProps),
    children
  );
};
/* eslint-enable react/prop-types */

var Table = function Table(_ref5) {
  var headerRows = _ref5.headerRows,
      bodyRows = _ref5.bodyRows,
      getRowId = _ref5.getRowId,
      columns = _ref5.columns,
      cellTemplate = _ref5.cellTemplate,
      onClick = _ref5.onClick;
  return React.createElement(TableLayout, {
    className: 'table-responsive',
    headerRows: headerRows,
    rows: bodyRows,
    getRowId: getRowId,
    columns: columns,
    minColumnWidth: MINIMAL_COLUMN_WIDTH,
    tableTemplate: tableTemplate$1,
    headTemplate: headTemplate,
    bodyTemplate: bodyTemplate,
    rowTemplate: rowTemplate,
    cellTemplate: cellTemplate,
    onClick: onClick
  });
};
Table.defaultProps = {
  onClick: function onClick() {}
};
Table.propTypes = {
  headerRows: PropTypes.array.isRequired,
  bodyRows: PropTypes.array.isRequired,
  columns: PropTypes.array.isRequired,
  cellTemplate: PropTypes.func.isRequired,
  getRowId: PropTypes.func.isRequired,
  onClick: PropTypes.func
};

var TableCell = function TableCell(_ref) {
  var style = _ref.style,
      row = _ref.row,
      column = _ref.column;
  return React.createElement(
    'td',
    {
      style: _extends({
        whiteSpace: 'nowrap',
        overflow: 'hidden',
        textOverflow: 'ellipsis',
        textAlign: column.align || 'left'
      }, style)
    },
    row[column.name]
  );
};

TableCell.propTypes = {
  style: PropTypes.shape(),
  row: PropTypes.shape(),
  column: PropTypes.shape()
};

TableCell.defaultProps = {
  style: null,
  row: {},
  column: {}
};

var TableNoDataCell = function TableNoDataCell(_ref) {
  var style = _ref.style,
      colspan = _ref.colspan;
  return React.createElement(
    'td',
    {
      style: _extends({
        textAlign: 'center',
        padding: '40px 0'
      }, style),
      colSpan: colspan
    },
    React.createElement(
      'big',
      { className: 'text-muted' },
      'No data'
    )
  );
};

TableNoDataCell.propTypes = {
  style: PropTypes.shape(),
  colspan: PropTypes.number
};

TableNoDataCell.defaultProps = {
  style: null,
  colspan: 1
};

var tableTemplate = function tableTemplate(props) {
  return React.createElement(Table, props);
};
var defaultCellTemplate = function defaultCellTemplate(props) {
  return React.createElement(TableCell, props);
};
var noDataCellTemplate = function noDataCellTemplate(props) {
  return React.createElement(TableNoDataCell, props);
};

var TableView$1 = function TableView$$1(_ref) {
  var tableCellTemplate = _ref.tableCellTemplate,
      props = objectWithoutProperties(_ref, ['tableCellTemplate']);
  return React.createElement(TableView, _extends({
    tableTemplate: tableTemplate,
    tableCellTemplate: combineTemplates(tableCellTemplate, defaultCellTemplate),
    tableNoDataCellTemplate: noDataCellTemplate
  }, props));
};
TableView$1.propTypes = {
  tableCellTemplate: PropTypes.func
};
TableView$1.defaultProps = {
  tableCellTemplate: undefined
};

/* global requestAnimationFrame cancelAnimationFrame */

var Sizer = function (_React$Component) {
  inherits(Sizer, _React$Component);

  function Sizer() {
    classCallCheck(this, Sizer);
    return possibleConstructorReturn(this, (Sizer.__proto__ || Object.getPrototypeOf(Sizer)).apply(this, arguments));
  }

  createClass(Sizer, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      var watchSizes = function watchSizes() {
        var rect = _this2.root.getBoundingClientRect();
        _this2.sizeUpdated(rect.width, rect.height);
        _this2._raf = requestAnimationFrame(watchSizes);
      };
      watchSizes();
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      cancelAnimationFrame(this._raf);
    }
  }, {
    key: 'sizeUpdated',
    value: function sizeUpdated(newWidth, newHeight) {
      var _props = this.props,
          height = _props.height,
          onHeightChange = _props.onHeightChange,
          width = _props.width,
          onWidthChange = _props.onWidthChange;


      if (height !== undefined && newHeight !== height) {
        onHeightChange(newHeight);
      }
      if (width !== undefined && newWidth !== width) {
        onWidthChange(newWidth);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      var _props2 = this.props,
          children = _props2.children,
          height = _props2.height,
          onHeightChange = _props2.onHeightChange,
          width = _props2.width,
          onWidthChange = _props2.onWidthChange,
          restProps = objectWithoutProperties(_props2, ['children', 'height', 'onHeightChange', 'width', 'onWidthChange']);


      return React.createElement(
        'div',
        _extends({ ref: function ref(_ref) {
            _this3.root = _ref;
          } }, restProps),
        children
      );
    }
  }]);
  return Sizer;
}(React.Component);
Sizer.defaultProps = {
  height: undefined,
  onHeightChange: function onHeightChange() {},
  width: undefined,
  onWidthChange: function onWidthChange() {}
};
Sizer.propTypes = {
  height: PropTypes.number,
  onHeightChange: PropTypes.func,
  width: PropTypes.number,
  onWidthChange: PropTypes.func,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]).isRequired
};

/* global navigator requestAnimationFrame */

var isSafari = /Safari/.test(navigator.userAgent) && /Apple Computer/.test(navigator.vendor);

var WindowedScroller = function (_React$Component) {
  inherits(WindowedScroller, _React$Component);

  function WindowedScroller(props) {
    classCallCheck(this, WindowedScroller);

    var _this = possibleConstructorReturn(this, (WindowedScroller.__proto__ || Object.getPrototypeOf(WindowedScroller)).call(this, props));

    _this.state = {
      viewport: { top: 0, left: 0, width: 0, height: 0 },
      offsetWidth: 0
    };

    _this.updateViewport = _this.updateViewport.bind(_this);
    return _this;
  }

  createClass(WindowedScroller, [{
    key: 'getChildContext',
    value: function getChildContext() {
      return {
        virtualHost: {
          viewport: this.state.viewport
        }
      };
    }
  }, {
    key: 'componentDidMount',
    value: function componentDidMount() {
      var _this2 = this;

      setTimeout(function () {
        _this2.updateViewport();
      });
    }
  }, {
    key: 'updateViewport',
    value: function updateViewport() {
      if (isSafari) {
        requestAnimationFrame(this._updateViewport.bind(this));
      } else {
        this._updateViewport();
      }
    }
  }, {
    key: '_updateViewport',
    value: function _updateViewport() {
      if (!this.root) return;

      var oldViewport = this.state.viewport;
      var viewport = {
        top: this.root.scrollTop,
        left: this.root.scrollLeft,
        width: this.root.clientWidth,
        height: this.root.clientHeight
      };

      // Prevent iOS to flicker in bounces =(
      if (viewport.top < 0 || viewport.left < 0 || viewport.left + viewport.width > this.root.scrollWidth || viewport.top + viewport.height > this.root.scrollHeight) {
        return;
      }

      // Optimize performance
      if (oldViewport.top !== viewport.top || oldViewport.left !== viewport.left || oldViewport.width !== viewport.width || oldViewport.height !== viewport.height) {
        this.setState({ viewport: viewport });
        this.props.onViewportChange(viewport);
      }
    }
  }, {
    key: 'render',
    value: function render() {
      var _this3 = this;

      return React.createElement(
        Sizer,
        {
          style: { width: '100%', height: '100%' },
          width: this.state.offsetWidth,
          onWidthChange: function onWidthChange(width) {
            _this3.setState({ offsetWidth: width });
            _this3.updateViewport();
          }
        },
        React.createElement(
          'div',
          {
            ref: function ref(_ref) {
              _this3.root = _ref;
            },
            onScroll: this.updateViewport,
            style: {
              overflow: 'auto',
              width: '100%',
              height: '100%',
              WebkitOverflowScrolling: 'touch',
              willChange: 'scroll-position'
            }
          },
          this.props.children
        )
      );
    }
  }]);
  return WindowedScroller;
}(React.Component);
WindowedScroller.defaultProps = {
  onViewportChange: function onViewportChange() {}
};
WindowedScroller.propTypes = {
  onViewportChange: PropTypes.func,
  children: PropTypes.oneOfType([PropTypes.node, PropTypes.arrayOf(PropTypes.node)]).isRequired
};
WindowedScroller.childContextTypes = {
  virtualHost: PropTypes.object.isRequired
};

/* global document */

function testCSSProp(property, value, noPrefixes) {
  var prop = property + ':';
  var el = document.createElement('test');
  var mStyle = el.style;

  if (!noPrefixes) {
    mStyle.cssText = prop + ['-webkit-', '-moz-', '-ms-', '-o-', ''].join(value + ';' + prop) + value + ';';
  } else {
    mStyle.cssText = prop + value;
  }
  return mStyle[property];
}
var stickyProp = testCSSProp('position', 'sticky');

function getVisibleItems(options) {
  var viewportStart = options.viewport.start;
  var viewportSize = options.viewport.size;

  var visibleItemMetas = [];
  var stickyItemsMetas = [];

  var index = 0;
  var offset = 0;
  while (index < options.itemCount) {
    var itemInfo = options.itemInfo(index);
    var itemSize = itemInfo.size;
    var itemStick = itemInfo.stick;

    if (itemStick === 'before' && offset <= viewportStart) {
      stickyItemsMetas.push({ index: index, offset: offset, size: itemSize, stick: itemStick });
      viewportStart += itemSize;
    }

    if ((offset + itemSize > viewportStart && offset + itemSize < viewportStart + viewportSize || offset < viewportStart + viewportSize && offset > viewportStart || offset <= viewportStart && offset + itemSize >= viewportStart + viewportSize) && itemSize > 0 && itemStick === false) {
      visibleItemMetas.push({ index: index, offset: offset, size: itemSize, stick: false });
    }

    index += 1;
    offset += itemSize;
  }

  return {
    visibleItemMetas: [].concat(stickyItemsMetas, visibleItemMetas),
    fullSize: offset,
    stickyBeforeSize: stickyItemsMetas.reduce(function (accumulator, meta) {
      return accumulator + meta.size;
    }, 0)
  };
}

var VirtualBox = function (_React$Component) {
  inherits(VirtualBox, _React$Component);

  function VirtualBox() {
    classCallCheck(this, VirtualBox);
    return possibleConstructorReturn(this, (VirtualBox.__proto__ || Object.getPrototypeOf(VirtualBox)).apply(this, arguments));
  }

  createClass(VirtualBox, [{
    key: 'getChildContext',
    value: function getChildContext() {
      var _viewport;

      var _props = this.props,
          direction = _props.direction,
          position = _props.position,
          stick = _props.stick;

      var viewport = this.context.virtualHost.viewport;

      var positionProp = direction === 'horizontal' ? 'left' : 'top';
      var crossPositionProp = direction === 'horizontal' ? 'top' : 'left';
      var sizeProp = direction === 'horizontal' ? 'width' : 'height';
      var crossSizeProp = direction === 'horizontal' ? 'height' : 'width';

      return {
        virtualHost: {
          viewport: (_viewport = {}, defineProperty(_viewport, positionProp, stick ? position : Math.max(viewport[positionProp] - position, 0)), defineProperty(_viewport, crossPositionProp, viewport[crossPositionProp]), defineProperty(_viewport, sizeProp, stick ? viewport[sizeProp] : Math.min(viewport[positionProp] + viewport[sizeProp] - position, viewport[sizeProp])), defineProperty(_viewport, crossSizeProp, viewport[crossSizeProp]), _viewport)
        }
      };
    }
  }, {
    key: 'getItemStyles',
    value: function getItemStyles(_ref) {
      var _babelHelpers$extends2;

      var position = _ref.position,
          size = _ref.size,
          stick = _ref.stick;
      var _props2 = this.props,
          direction = _props2.direction,
          crossSize = _props2.crossSize;

      var viewport = this.context.virtualHost.viewport;

      var positionProp = direction === 'horizontal' ? 'left' : 'top';
      var sizeProp = direction === 'horizontal' ? 'width' : 'height';
      var crossSizeProp = direction === 'vertical' ? 'width' : 'height';

      var styles = defineProperty({
        position: 'absolute'
      }, positionProp, position + 'px');

      if (stick) {
        if (stickyProp) {
          styles = _extends({}, styles, defineProperty({

            zIndex: 1,
            position: stickyProp
          }, positionProp, position + (viewport[positionProp + 'Stick'] || 0) + 'px'));
        } else if (direction === 'vertical') {
          var _styles2;

          styles = (_styles2 = {
            zIndex: 1,
            position: 'fixed'
          }, defineProperty(_styles2, crossSizeProp, viewport[crossSizeProp] + 'px'), defineProperty(_styles2, 'overflow', 'hidden'), defineProperty(_styles2, 'WebkitBackfaceVisibility', 'hidden'), defineProperty(_styles2, 'backfaceVisibility', 'hidden'), _styles2);
          /* children = (
              <div
                style={{
                  marginLeft: -viewport[crossPositionProp] + 'px',
                  width: '100%',
                  height: '100%'
                }}
              >
                  {children}
              </div>
          );*/
        }
      }

      return _extends({}, styles, (_babelHelpers$extends2 = {
        display: 'block'
      }, defineProperty(_babelHelpers$extends2, sizeProp, size + 'px'), defineProperty(_babelHelpers$extends2, crossSizeProp, crossSize + 'px'), _babelHelpers$extends2));
    }
  }, {
    key: 'render',
    value: function render() {
      var _this2 = this,
          _babelHelpers$extends3;

      var _props3 = this.props,
          direction = _props3.direction,
          position = _props3.position,
          stick = _props3.stick,
          iref = _props3.iref;

      var viewport = this.context.virtualHost.viewport;

      var positionProp = direction === 'horizontal' ? 'left' : 'top';
      var sizeProp = direction === 'horizontal' ? 'width' : 'height';

      var _getVisibleItems = getVisibleItems({
        viewport: { start: stick ? position : viewport[positionProp], size: viewport[sizeProp] },
        itemCount: this.props.itemCount,
        itemInfo: this.props.itemInfo
      }),
          visibleItemMetas = _getVisibleItems.visibleItemMetas,
          fullSize = _getVisibleItems.fullSize;

      var visibleItems = visibleItemMetas.map(function (visibleItemMeta) {
        var styles = _this2.getItemStyles({
          position: visibleItemMeta.offset,
          size: visibleItemMeta.size,
          stick: visibleItemMeta.stick
        });

        return React.cloneElement(_this2.props.itemTemplate(visibleItemMeta.index, visibleItemMeta.offset, styles), {
          key: '' + visibleItemMeta.index,
          style: styles
        });
      });

      var RootTag = this.props.rootTag;

      return React.createElement(
        RootTag,
        _extends({
          className: this.props.className,
          style: _extends({
            position: 'relative'
          }, this.props.style, (_babelHelpers$extends3 = {}, defineProperty(_babelHelpers$extends3, sizeProp, fullSize + 'px'), defineProperty(_babelHelpers$extends3, 'display', 'block'), _babelHelpers$extends3))
        }, iref ? { ref: iref } : {}),
        visibleItems
      );
    }
  }]);
  return VirtualBox;
}(React.Component);
VirtualBox.defaultProps = {
  rootTag: 'div',
  className: '',
  style: {},
  position: 0,
  stick: false,
  crossSize: 0,
  iref: undefined
};
VirtualBox.propTypes = {
  rootTag: PropTypes.string,
  className: PropTypes.string,
  style: PropTypes.object,
  position: PropTypes.number,
  stick: PropTypes.bool,
  iref: PropTypes.func,
  crossSize: PropTypes.number,
  direction: PropTypes.oneOf(['vertical', 'horizontal']).isRequired,
  itemCount: PropTypes.number.isRequired,
  itemInfo: PropTypes.func.isRequired,
  itemTemplate: PropTypes.func.isRequired
};
VirtualBox.contextTypes = {
  virtualHost: PropTypes.object
};
VirtualBox.childContextTypes = {
  virtualHost: PropTypes.object.isRequired
};

/* global window document */

var DEFAULT_HEIGHT = 37;
var MINIMAL_COLUMN_WIDTH$1 = 120;

var calculateColumnWidths = function calculateColumnWidths(columns, tableWidth) {
  var occupiedWidth = columns.map(function (column) {
    return column.width;
  }).filter(function (width) {
    return !!width;
  }).reduce(function (accum, width) {
    return accum + width;
  }, 0);
  var restWidthSpread = columns.map(function (column) {
    return column.width;
  }).filter(function (width) {
    return !width;
  }).length;
  var calculatedWidth = (tableWidth - occupiedWidth) / restWidthSpread;
  var widthForAutoColumn = Math.max(calculatedWidth, MINIMAL_COLUMN_WIDTH$1);

  return columns.map(function (column) {
    return column.width;
  }).map(function (width) {
    return !!width && width !== 'auto' ? width : widthForAutoColumn;
  });
};

var VirtualTable = function (_React$Component) {
  inherits(VirtualTable, _React$Component);

  function VirtualTable(props) {
    classCallCheck(this, VirtualTable);

    var _this = possibleConstructorReturn(this, (VirtualTable.__proto__ || Object.getPrototypeOf(VirtualTable)).call(this, props));

    _this.state = {
      viewportWidth: 0,
      autoHeights: new WeakMap()
    };

    _this.rowHeight = function (row) {
      var height = row.height || DEFAULT_HEIGHT;
      if (height === 'auto') {
        height = DEFAULT_HEIGHT;
      }
      return height;
    };
    return _this;
  }

  createClass(VirtualTable, [{
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          headerRows = _props.headerRows,
          bodyRows = _props.bodyRows,
          columns = _props.columns,
          cellTemplate = _props.cellTemplate;


      var columnWidths = calculateColumnWidths(columns, this.state.viewportWidth);
      var scrollWidth = columnWidths.reduce(function (accum, width) {
        return accum + width;
      }, 0);

      var tableRow = function tableRow(row, position) {
        var colspan = row.colspan;
        var columnLength = colspan !== undefined ? colspan + 1 : columns.length;
        return React.createElement(VirtualBox, {
          rootTag: 'tr',

          position: position,
          crossSize: _this2.rowHeight(row),
          direction: 'horizontal',
          itemCount: columnLength,
          itemInfo: function itemInfo(columnIndex) {
            var size = columnIndex !== colspan ? columnWidths[columnIndex] : columns.slice(colspan).reduce(function (accum, column) {
              return accum + columnWidths[columns.indexOf(column)];
            }, 0);

            return {
              size: size,
              stick: false
            };
          },
          itemTemplate: function itemTemplate(columnIndex, _, style) {
            return cellTemplate({ row: row, column: columns[columnIndex], style: style });
          }
        });
      };

      var tableHead = function tableHead(position) {
        return React.createElement(VirtualBox, {
          key: 'thead',
          rootTag: 'thead',

          iref: function iref(ref) {
            if (!ref) return;
            var style = ref.style;

            style.backgroundColor = window.getComputedStyle(document.body).backgroundColor;
          },

          position: position,
          stick: true,
          direction: 'vertical',
          itemCount: headerRows.length,
          itemInfo: function itemInfo(rowIndex) {
            return {
              size: _this2.rowHeight(headerRows[rowIndex]),
              stick: false
            };
          },
          itemTemplate: function itemTemplate(rowIndex, rowPosition) {
            return tableRow(headerRows[rowIndex], rowPosition);
          }
        });
      };

      var tableBody = function tableBody(position) {
        return React.createElement(VirtualBox, {
          key: 'tbody',
          rootTag: 'tbody',

          position: position,
          direction: 'vertical',
          itemCount: bodyRows.length,
          itemInfo: function itemInfo(rowIndex) {
            return {
              size: _this2.rowHeight(bodyRows[rowIndex]),
              stick: false
            };
          },
          itemTemplate: function itemTemplate(rowIndex, rowPosition) {
            return tableRow(bodyRows[rowIndex], rowPosition);
          }
        });
      };

      return React.createElement(
        'div',
        {
          style: { height: '530px' }
        },
        React.createElement(
          WindowedScroller,
          {
            onViewportChange: function onViewportChange(viewport) {
              return _this2.setState({ viewportWidth: viewport.width });
            }
          },
          React.createElement(VirtualBox, {
            rootTag: 'table',
            className: 'table',
            style: { zIndex: 0, margin: 0 },

            crossSize: scrollWidth,
            direction: 'vertical',
            itemCount: 2,
            itemInfo: function itemInfo(rowIndex) {
              var size = rowIndex === 0 ? headerRows.reduce(function (accum, row) {
                return accum + _this2.rowHeight(row);
              }, 0) : bodyRows.reduce(function (accum, row) {
                return accum + _this2.rowHeight(row);
              }, 0);

              return {
                size: size,
                stick: rowIndex === 0 ? 'before' : false
              };
            },
            itemTemplate: function itemTemplate(rowIndex, position) {
              return rowIndex === 0 ? tableHead(position) : tableBody(position);
            }
          })
        )
      );
    }
  }]);
  return VirtualTable;
}(React.Component);

VirtualTable.propTypes = {
  headerRows: PropTypes.array.isRequired,
  bodyRows: PropTypes.array.isRequired,
  columns: PropTypes.array.isRequired,
  cellTemplate: PropTypes.func.isRequired
};

var tableTemplate$2 = function tableTemplate(props) {
  return React.createElement(VirtualTable, props);
};
var defaultCellTemplate$1 = function defaultCellTemplate(props) {
  return React.createElement(TableCell, props);
};
var noDataCellTemplate$1 = function noDataCellTemplate(props) {
  return React.createElement(TableNoDataCell, props);
};

var VirtualTableView = function VirtualTableView(_ref) {
  var tableCellTemplate = _ref.tableCellTemplate,
      props = objectWithoutProperties(_ref, ['tableCellTemplate']);
  return React.createElement(TableView, _extends({
    tableTemplate: tableTemplate$2,
    tableCellTemplate: combineTemplates(tableCellTemplate, defaultCellTemplate$1),
    tableNoDataCellTemplate: noDataCellTemplate$1
  }, props));
};
VirtualTableView.propTypes = {
  tableCellTemplate: PropTypes.func
};
VirtualTableView.defaultProps = {
  tableCellTemplate: undefined
};

var TableFilterCell = function TableFilterCell(_ref) {
  var style = _ref.style,
      column = _ref.column,
      filter = _ref.filter,
      setFilter = _ref.setFilter;
  return React.createElement(
    'th',
    {
      style: _extends({
        fontWeight: 'normal'
      }, style)
    },
    !column.type && React.createElement('input', {
      type: 'text',
      className: 'form-control',
      value: filter ? filter.value : '',
      onChange: function onChange(e) {
        return setFilter({ value: e.target.value });
      }
    })
  );
};

TableFilterCell.propTypes = {
  column: PropTypes.object,
  style: PropTypes.object,
  filter: PropTypes.object,
  setFilter: PropTypes.func
};

TableFilterCell.defaultProps = {
  column: {},
  style: null,
  filter: null,
  setFilter: function setFilter() {}
};

var defaultFilterCellTemplate = function defaultFilterCellTemplate(props) {
  return React.createElement(TableFilterCell, props);
};

var TableFilterRow$1 = function TableFilterRow$$1(_ref) {
  var filterCellTemplate = _ref.filterCellTemplate,
      restProps = objectWithoutProperties(_ref, ['filterCellTemplate']);
  return React.createElement(TableFilterRow, _extends({
    filterCellTemplate: combineTemplates(filterCellTemplate, defaultFilterCellTemplate)
  }, restProps));
};

TableFilterRow$1.propTypes = {
  filterCellTemplate: PropTypes.func
};
TableFilterRow$1.defaultProps = {
  filterCellTemplate: undefined
};

var TableHeaderCell = function TableHeaderCell(_ref) {
  var _ref2;

  var style = _ref.style,
      column = _ref.column,
      allowSorting = _ref.allowSorting,
      sortingDirection = _ref.sortingDirection,
      changeSortingDirection = _ref.changeSortingDirection,
      allowGrouping = _ref.allowGrouping,
      groupByColumn = _ref.groupByColumn,
      allowDragging = _ref.allowDragging,
      dragPayload = _ref.dragPayload;

  var align = column.align || 'left';
  var invertedAlign = align === 'left' ? 'right' : 'left';
  var columnTitle = column.title || column.name;

  var gropingControl = allowGrouping && React.createElement(
    'div',
    {
      onClick: function onClick(e) {
        e.stopPropagation();
        groupByColumn(e);
      },
      style: {
        float: invertedAlign,
        textAlign: invertedAlign,
        width: '14px'
      }
    },
    React.createElement('i', {
      className: 'glyphicon glyphicon-th-list',
      style: {
        top: '0',
        fontSize: '9px',
        margin: '-5px',
        padding: '5px'
      }
    })
  );

  var sortingControl = allowSorting && (align === 'right' ? React.createElement(
    'span',
    {
      className: sortingDirection ? 'text-primary' : ''
    },
    React.createElement(SortingIndicator, {
      direction: sortingDirection,
      style: { visibility: sortingDirection ? 'visible' : 'hidden' }
    }),
    '\xA0',
    columnTitle
  ) : React.createElement(
    'span',
    {
      className: sortingDirection ? 'text-primary' : ''
    },
    columnTitle,
    '\xA0',
    React.createElement(SortingIndicator, {
      direction: sortingDirection,
      style: { visibility: sortingDirection ? 'visible' : 'hidden' }
    })
  ));

  var cellLayout = React.createElement(
    'th',
    {
      style: _extends({
        userSelect: 'none',
        MozUserSelect: 'none',
        WebkitUserSelect: 'none',
        cursor: allowSorting && !column.type ? 'pointer' : 'default'
      }, style),
      onClick: function onClick(e) {
        if (!allowSorting) return;
        e.stopPropagation();
        var cancelSortingRelatedKey = e.metaKey || e.ctrlKey;
        changeSortingDirection({
          keepOther: e.shiftKey || cancelSortingRelatedKey,
          cancel: cancelSortingRelatedKey
        });
      }
    },
    gropingControl,
    React.createElement(
      'div',
      {
        style: (_ref2 = {}, defineProperty(_ref2, 'margin' + (column.align === 'right' ? 'Left' : 'Right'), '14px'), defineProperty(_ref2, 'textAlign', align), defineProperty(_ref2, 'whiteSpace', 'nowrap'), defineProperty(_ref2, 'overflow', 'hidden'), defineProperty(_ref2, 'textOverflow', 'ellipsis'), _ref2)
      },
      allowSorting ? sortingControl : columnTitle
    )
  );

  return allowDragging ? React.createElement(
    DragSource,
    { getPayload: function getPayload() {
        return dragPayload;
      } },
    cellLayout
  ) : cellLayout;
};

TableHeaderCell.propTypes = {
  column: PropTypes.shape({
    title: PropTypes.string
  }).isRequired,
  style: PropTypes.shape(),
  allowSorting: PropTypes.bool,
  sortingDirection: PropTypes.oneOf(['asc', 'desc', null]),
  changeSortingDirection: PropTypes.func,
  allowGrouping: PropTypes.bool,
  groupByColumn: PropTypes.func,
  allowDragging: PropTypes.bool,
  dragPayload: PropTypes.any
};

TableHeaderCell.defaultProps = {
  style: null,
  allowSorting: false,
  sortingDirection: undefined,
  changeSortingDirection: undefined,
  allowGrouping: false,
  groupByColumn: undefined,
  allowDragging: false,
  dragPayload: null
};

var headerCellTemplate = function headerCellTemplate(props) {
  return React.createElement(TableHeaderCell, props);
};

var TableHeaderRow$1 = function TableHeaderRow$$1(props) {
  return React.createElement(TableHeaderRow, _extends({
    headerCellTemplate: headerCellTemplate
  }, props));
};

var EditCell = function EditCell(_ref) {
  var column = _ref.column,
      value = _ref.value,
      onValueChange = _ref.onValueChange;
  return React.createElement(
    'td',
    {
      style: {
        verticalAlign: 'middle',
        padding: '1px'
      }
    },
    React.createElement('input', {
      type: 'text',
      className: 'form-control',
      value: value,
      onChange: function onChange(e) {
        return onValueChange(e.target.value);
      },
      style: { width: '100%', textAlign: column.align }
    })
  );
};
EditCell.propTypes = {
  column: PropTypes.object.isRequired,
  value: PropTypes.any,
  onValueChange: PropTypes.func.isRequired
};
EditCell.defaultProps = {
  value: undefined
};

var defaultEditCellTemplate = function defaultEditCellTemplate(props) {
  return React.createElement(EditCell, props);
};

var TableEditRow$1 = function TableEditRow$$1(_ref) {
  var editCellTemplate = _ref.editCellTemplate,
      restProps = objectWithoutProperties(_ref, ['editCellTemplate']);
  return React.createElement(TableEditRow, _extends({
    editCellTemplate: combineTemplates(editCellTemplate, defaultEditCellTemplate)
  }, restProps));
};
TableEditRow$1.propTypes = {
  editCellTemplate: PropTypes.func
};
TableEditRow$1.defaultProps = {
  editCellTemplate: undefined
};

var CommandButton = function CommandButton(_ref) {
  var executeCommand = _ref.executeCommand,
      text = _ref.text;
  return React.createElement(
    'button',
    {
      className: 'btn btn-link',
      onClick: function onClick(e) {
        executeCommand();
        e.stopPropagation();
      }
    },
    text
  );
};
CommandButton.propTypes = {
  executeCommand: PropTypes.func.isRequired,
  text: PropTypes.func.isRequired
};

var EditCommandHeadingCell = function EditCommandHeadingCell(_ref2) {
  var addRow = _ref2.addRow,
      commandTemplate = _ref2.commandTemplate,
      allowAdding = _ref2.allowAdding,
      _ref2$style = _ref2.style,
      style = _ref2$style === undefined ? {} : _ref2$style,
      _ref2$addCommandText = _ref2.addCommandText,
      addCommandText = _ref2$addCommandText === undefined ? 'New' : _ref2$addCommandText;
  return React.createElement(
    'th',
    {
      style: _extends({
        whiteSpace: 'nowrap',
        textAlign: 'center',
        padding: 0
      }, style)
    },
    allowAdding && commandTemplate({
      id: 'add',
      executeCommand: addRow,
      text: addCommandText
    })
  );
};
EditCommandHeadingCell.propTypes = {
  addRow: PropTypes.func.isRequired,
  commandTemplate: PropTypes.func.isRequired,
  addCommandText: PropTypes.string.isRequired,
  allowAdding: PropTypes.bool.isRequired,
  style: PropTypes.object
};
EditCommandHeadingCell.defaultProps = {
  style: undefined
};

var EditCommandCell = function EditCommandCell(_ref3) {
  var startEditing = _ref3.startEditing,
      deleteRow = _ref3.deleteRow,
      cancelEditing = _ref3.cancelEditing,
      commitChanges = _ref3.commitChanges,
      isEditing = _ref3.isEditing,
      commandTemplate = _ref3.commandTemplate,
      allowEditing = _ref3.allowEditing,
      allowDeleting = _ref3.allowDeleting,
      _ref3$style = _ref3.style,
      style = _ref3$style === undefined ? {} : _ref3$style,
      _ref3$editCommandText = _ref3.editCommandText,
      editCommandText = _ref3$editCommandText === undefined ? 'Edit' : _ref3$editCommandText,
      _ref3$deleteCommandTe = _ref3.deleteCommandText,
      deleteCommandText = _ref3$deleteCommandTe === undefined ? 'Delete' : _ref3$deleteCommandTe,
      _ref3$commitCommandTe = _ref3.commitCommandText,
      commitCommandText = _ref3$commitCommandTe === undefined ? 'Save' : _ref3$commitCommandTe,
      _ref3$cancelCommandTe = _ref3.cancelCommandText,
      cancelCommandText = _ref3$cancelCommandTe === undefined ? 'Cancel' : _ref3$cancelCommandTe;

  var commands = [];
  if (!isEditing) {
    if (allowEditing) {
      commands.push({
        id: 'edit',
        executeCommand: startEditing,
        text: editCommandText
      });
    }
    if (allowDeleting) {
      commands.push({
        id: 'delete',
        executeCommand: deleteRow,
        text: deleteCommandText
      });
    }
  } else {
    commands = [{
      id: 'commit',
      executeCommand: commitChanges,
      text: commitCommandText
    }, {
      id: 'cancel',
      executeCommand: cancelEditing,
      text: cancelCommandText
    }];
  }
  return React.createElement(
    'td',
    {
      style: _extends({
        whiteSpace: 'nowrap',
        textAlign: 'center',
        padding: 0
      }, style)
    },
    commands.map(function (command) {
      return React.createElement(
        'span',
        { key: command.id },
        commandTemplate(command)
      );
    })
  );
};
EditCommandCell.propTypes = {
  startEditing: PropTypes.func.isRequired,
  deleteRow: PropTypes.func.isRequired,
  cancelEditing: PropTypes.func.isRequired,
  commitChanges: PropTypes.func.isRequired,
  isEditing: PropTypes.bool.isRequired,
  allowEditing: PropTypes.bool.isRequired,
  allowDeleting: PropTypes.bool.isRequired,
  commandTemplate: PropTypes.func.isRequired,
  editCommandText: PropTypes.string.isRequired,
  deleteCommandText: PropTypes.string.isRequired,
  commitCommandText: PropTypes.string.isRequired,
  cancelCommandText: PropTypes.string.isRequired,
  style: PropTypes.object
};
EditCommandCell.defaultProps = {
  style: undefined
};

var TableEditColumn$1 = function TableEditColumn$$1(_ref) {
  var cellTemplate = _ref.cellTemplate,
      headingCellTemplate = _ref.headingCellTemplate,
      commandTemplate = _ref.commandTemplate,
      restProps = objectWithoutProperties(_ref, ['cellTemplate', 'headingCellTemplate', 'commandTemplate']);
  return React.createElement(TableEditColumn, _extends({}, restProps, {
    cellTemplate: combineTemplates(cellTemplate, EditCommandCell),
    headingCellTemplate: combineTemplates(headingCellTemplate, EditCommandHeadingCell),
    commandTemplate: combineTemplates(commandTemplate, CommandButton)
  }));
};
TableEditColumn$1.propTypes = {
  cellTemplate: PropTypes.func,
  headingCellTemplate: PropTypes.func,
  commandTemplate: PropTypes.func
};
TableEditColumn$1.defaultProps = {
  cellTemplate: undefined,
  headingCellTemplate: undefined,
  commandTemplate: undefined
};

export { Grid$1 as Grid, DragDropContext$1 as DragDropContext, PagingPanel$1 as PagingPanel, GroupingPanel$1 as GroupingPanel, TableRowDetail$1 as TableRowDetail, TableGroupRow$1 as TableGroupRow, TableSelection$1 as TableSelection, TableView$1 as TableView, VirtualTableView, TableFilterRow$1 as TableFilterRow, TableHeaderRow$1 as TableHeaderRow, TableEditRow$1 as TableEditRow, TableEditColumn$1 as TableEditColumn };
//# sourceMappingURL=dx-react-grid-bootstrap3.es.js.map
